from django.shortcuts import render
from django.db.models import Count, Q , Max , Min
from rest_framework.views import APIView
from datetime import timedelta
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import user_passes_test
from django.db.models.functions import Coalesce
from rest_framework.response import Response
from rest_framework import status

from clmainApp.models import Branch,User,PatientStatus
from brsessionApp.models import SessionStatus,PatientSessionItem
from brsessionApp.serializers import PsiAssessmentSerializer,PsiAssessment
from brpatientApp.models import PatientInfo
from clmainApp.utils import getStartEndDateTime
import operator
from functools import reduce
from django.db.models import Sum

class DoctorsSessionsReportApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_DoctorsSessionsReport'), login_url='/main/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values("id","name")
        session_status = SessionStatus.objects.all().order_by("code")
        return render(request,"Clinic/clreportApp/doctors_sessions_report.html",{
            #"locations_names":locations_names,"locations_count":locations_count,
            "branches":branches,"session_status":session_status,
            'activePage':'DoctorsSessionsReport'})
    
    def post(self,request):
        data=request.data
        data._mutable=True
        startDate = data["startDate"]
        endDate = data["endDate"]
        branch_id = data["branch_id"]
        branch_doctors = User.objects.filter(user_category="2DR",groups__branch_id=branch_id,is_active=True).values("id","name")
        branch_doctors_ids__ = branch_doctors.values_list("id",flat=True)
        branch_doctors_ids = list(set(branch_doctors_ids__))
        session_status = SessionStatus.objects.all().values("code","color_code").order_by("code")
        #q_list = [Q(session_day__range=[lastPeriodstartDateTime,lastPeriodendDateTime])]
        ps_items = PatientSessionItem.objects.filter(session__patient__branch_id=branch_id,doctor_id__in=branch_doctors_ids,session_day__range=[startDate,endDate]).values("id","confirmed_by_id","doctor_id","session_status_id","session__patient_id",'assessment_id','assessment_s')
        # جميع الجلسات خلال الفترة اللى متاخدش عليها اجراء 
        psi_sessions_without_B_and_D__data = ps_items.filter(psi_assessment_choice="UD",psi_assessment_choice_s="UD")
        psi_sessions_without_B_and_D_list_ids = psi_sessions_without_B_and_D__data.values_list('id',flat=True)
        #جميع الجلسات خلال الفترة  خدو عليها بونص او خصم  
        psi_sessions_with_B_and_D__data = ps_items.exclude(id__in=psi_sessions_without_B_and_D_list_ids)
        psi_sessions_with_B_and_D__data_patients = set(psi_sessions_with_B_and_D__data.values_list('session__patient_id',flat=True))
        #old way to calc assessment data 
        '''
        psi_assessments_ids_all = psi_sessions_with_B_and_D__data.values('assessment_id','assessment_s')
        psi_assessments_ids = []
        for assessment_obj in psi_assessments_ids_all :
            if not assessment_obj["assessment_id"] and not assessment_obj["assessment_s"] : continue
            else :
                if assessment_obj["assessment_id"] : psi_assessments_ids.append(assessment_obj["assessment_id"])
                if assessment_obj["assessment_s"] : psi_assessments_ids.append(assessment_obj["assessment_s"])
        #psi_assessments_ids = [ assessment_obj["assessment_id"] if (not assessment_obj["assessment_s"]) else  assessment_obj["assessment_s"] for assessment_obj in psi_assessments_ids_all]
        #for assessment_obj in psi_assessments_ids_all :
        #assessment_id = assessment_obj["assessment_id"] if (not assessment_obj["assessment_s"]) else  assessment_obj["assessment_s"]
        psi_assessments_ids_without_None = set(psi_assessments_ids)#still contain None , next line will remove NOne 
        psi_assessments_ids_without_None.discard(None)
        psi_assessments = PsiAssessment.objects.filter(id__in=psi_assessments_ids_without_None).values('id','doctor','assessment_choice','sessions_count')
        '''
        psi_assessments = PsiAssessment.objects.filter(patient__branch_id=branch_id,created_at__date__range=[startDate,endDate]).values('id','doctor','assessment_choice','sessions_count')
        response_data =[]
        for dr_id in branch_doctors_ids:
            dr_obj ={"dr_id":dr_id,"dr_name":branch_doctors.filter(id=dr_id)[0]["name"]}
            #dr_sessions = ps_items.filter(confirmed_by_id=dr_id)
            dr_sessions = ps_items.filter(doctor_id=dr_id)
            dr_obj["dr_sessions_list"] = dr_sessions.values_list('id',flat=True)

            dr_assessments = psi_assessments.filter(doctor_id=dr_id)

            dr_Bonus_list = dr_assessments.filter(assessment_choice="B")
            dr_Bonus_list_ids = dr_Bonus_list.values_list('id',flat=True)
            dr_obj["dr_Bonus_list"] = dr_Bonus_list_ids
            #dr_obj["dr_Bonus"]  = ps_items.filter(Q(assessment_id__in=dr_Bonus_list_ids)|Q(assessment_s_id__in=dr_Bonus_list_ids)).count()
            dr_obj["dr_Bonus"]  = dr_Bonus_list.aggregate( total_sessions=Coalesce(Sum('sessions_count'), 0) )['total_sessions']
            dr_Deduction_list = dr_assessments.filter(assessment_choice="D")
            dr_Deduction_list_ids = dr_Deduction_list.values_list('id',flat=True)
            dr_obj["dr_Deduction_list"] =dr_Deduction_list_ids
            # dr_sessions will be -- wrong -- because he may take bouns or deduction for other doctor sessions 
            ## dr_obj["dr_Deduction"] = dr_sessions.filter( assessment_id__in=dr_Deduction_list_ids).values().count()
            # good practice should be filter with all sessions in period 
            #dr_obj["dr_Deduction"] = ps_items.filter(Q(assessment_id__in=dr_Deduction_list_ids)|Q(assessment_s_id__in=dr_Deduction_list_ids)).count()
            dr_obj["dr_Deduction"]  = dr_Deduction_list.aggregate( total_sessions=Coalesce(Sum('sessions_count'), 0) )['total_sessions']

            #dr_obj["dr_Deduction"] = dr_sessions.filter(Q(psi_assessment_choice="D", assessment__doctor_id=dr_id,assessment__assessment_code="P") | Q(psi_assessment_choice_s="D", assessment_s__doctor_id=dr_id,assessment__assessment_code="S")).count()
            ##dr_obj["dr_Bonus"]  = ps_items.filter(Q(psi_assessment_choice="B", assessment__doctor_id=dr_id) | Q(psi_assessment_choice_s="B", assessment_s__doctor_id=dr_id)).count()
            #dr_obj["dr_Bonus"] =  dr_sessions.filter(reduce(operator.and_,[Q(psi_assessment_choice="B"),Q(assessment__doctor_id=dr_id)]), reduce(operator.or_, [     Q(assessment__doctor_id=dr_id),Q(assessment_s__doctor_id=dr_id)]))
            #dr_obj["dr_Bonus"] = dr_sessions.filter(reduce(operator.or_, [Q(psi_assessment_choice="B"),Q(psi_assessment_choice_s="B")])).exclude(~Q(assessment__doctor_id=dr_id)).exclude(~Q(assessment_s__doctor_id=dr_id)).count()
            #dr_obj["dr_Deduction"] = dr_sessions.filter(reduce(operator.or_, [Q(psi_assessment_choice="D"),Q(psi_assessment_choice_s="D")])).exclude(~Q(assessment__doctor_id=dr_id)).exclude(~Q(assessment_s__doctor_id=dr_id)).count()
            #dr_obj["dr_Bonus"] = dr_Bonus_list.aggregate(total_Bonus=Coalesce(Sum('sessions_count'),0))["total_Bonus"]
            #dr_obj["dr_Deduction"] = dr_Deduction_list.aggregate(total_Deduction=Coalesce(Sum('sessions_count'),0))["total_Deduction"]
            dr_sessions_total = dr_sessions.count()
            dr_obj["dr_sessions_total"] = dr_sessions_total
            dr_obj["dr_sessions_confirmed"] = ps_items.filter(confirmed_by_id=dr_id).count()
            #dr_sessions_without_B_and_D_list
            #dr_sessions_B_and_D__ids = [psi_id for psi_id in dr_obj["dr_Bonus_list"]]  +  [psi_id for psi_id in dr_obj["dr_Deduction_list"]]
            dr_sessions_without_B_and_D__data = dr_sessions.filter(psi_assessment_choice="UD",psi_assessment_choice_s="UD")
            dr_sessions_without_B_and_D_list_ids = dr_sessions_without_B_and_D__data.values_list('id',flat=True)
            #المرضى اللى عندهم جلسات متاخدش عليهم اجراء 
            patients_ids = set(dr_sessions_without_B_and_D_list_ids.values_list('session__patient_id',flat=True))
            dr_obj["dr_sessions_without_B_and_D_pt_list"] = [pt_id for pt_id in patients_ids]
            dr_sessions_without_B_and_D = dr_sessions_without_B_and_D_list_ids.count()
            dr_obj["dr_sessions_without_B_and_D"] = dr_sessions_without_B_and_D
            # import pdb
            # pdb.set_trace()
            #المرضى اللى عندهم جلسات   خدو عليها بونص او خصم  
            #dr_sessions_with_B_and_D__data_patients = set(dr_sessions.exclude(id__in=dr_sessions_without_B_and_D_list_ids).values_list('session__patient_id',flat=True))
            #import pdb
            #pdb.set_trace()
            #المرضى اللى عندهم جلسات  متاخدش اى اجراء على كل جلساتهم خلال الفترة   
            patients_without_B_and_D_at_all = patients_ids - psi_sessions_with_B_and_D__data_patients
            dr_obj["patients_without_B_and_D_at_all_list"] = [pt_id for pt_id in patients_without_B_and_D_at_all]
            # عدد الجلسات للمرضى اللى متاخدش عندهم اى اجراء خلال الفترة
            dr_obj["sessions_without_B_and_D_along_period"] = dr_sessions.filter(session__patient_id__in=patients_without_B_and_D_at_all).count()
            #الجلسات الملغية وحالات الجلسات
            canceled_sessions_ids = dr_sessions.filter(session_status_id="6CL").values_list('id',flat=True)
            dr_obj["canceled_sessions_list"]=canceled_sessions_ids
            dr_sessions_detail_list = []
            for status_obj in session_status :
                #status_obj_info = {}
                status_obj_code = status_obj["code"]
                #status_obj_color_code = status_obj["color_code"]
                status_obj_count = dr_sessions.filter(session_status_id=status_obj_code).values('id').aggregate(count_sessions=Coalesce(Count('id'),0))['count_sessions']
                dr_sessions_detail_list.append(status_obj_count)
            dr_obj["dr_sessions_detail_list"] = dr_sessions_detail_list
            response_data.append(dr_obj)
        return Response(response_data , status=status.HTTP_201_CREATED)

# class CanceledVisitsCCReport(APIView):
#     @method_decorator(user_passes_test(lambda u: u.has_perm('screen.view_canceled_visits_cc_report'), login_url='/main/login/'))
#     def get(self,request):
#         cancel_reasons = LeadVisit.objects.filter(cancel_reason_id__gt=0).values("cancel_reason__name").annotate(Count('cancel_reason'))
#         #locations_names = [l['location__name'] for l in locations]
#         #locations_count = [l['location__count'] for l in locations]
#         cancel_reasons_names = [w['cancel_reason__name'] for w in cancel_reasons]
#         cancel_reasons_count = [w['cancel_reason__count'] for w in cancel_reasons]
#         return render(request,"Clinic/clreportApp/canceled_visits_cc_report.html",{
#             #"locations_names":locations_names,"locations_count":locations_count,
#             "cancel_reasons_names":cancel_reasons_names,"cancel_reasons_count":cancel_reasons_count,
#             'activePage':'CanceledVisitsCCReport'})

class PsiAssessmentsDataApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('screen.view_canceled_visits_cc_report'), login_url='/main/login/'))
    def post(self,request):
        data = request.data
        assessments_ids = data.getlist('assessments_ids[]')
        #startDate = data["startDate"]
        #endDate = data["endDate"]
        assessments = PsiAssessment.objects.filter(id__in=assessments_ids).order_by('patient_id','assessment_code')
        assessments_ids = assessments.values_list("id",flat=True)
        #ps_items = PatientSessionItem.objects.filter(reduce(operator.or_, [Q(assessment_id__in=assessments_ids),Q(assessment_s_id__in=assessments_ids)]),session_day__range=[startDate,endDate]).values("id")
        serializer = PsiAssessmentSerializer(assessments,many=True)
        data = serializer.data
        #for ass_obj in data :ass_obj["sessions_count"]=ps_items.filter(reduce(operator.or_, [Q(assessment_id=ass_obj["id"]),Q(assessment_s_id=ass_obj["id"])])).count()
        return Response( data, status=status.HTTP_200_OK)
    
from collections import Counter
class PsiWithoutAssessmentsDataApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('screen.view_canceled_visits_cc_report'), login_url='/main/login/'))
    def post(self,request):
        data = request.data
        pt_ids = data.getlist('pt_ids[]')
        dr_id = data["dr_id"]
        startDate = data["startDate"]
        endDate = data["endDate"]
        ps_items = PatientSessionItem.objects.filter(doctor_id=dr_id,session__patient_id__in=pt_ids,psi_assessment_choice="UD",psi_assessment_choice_s="UD",session_day__range=[startDate,endDate]).values("session__patient__fullname","session__patient__code","session__patient_id")
        patient__code_values = list(ps_items.values_list("session__patient__code", flat=True))
        patient__code_counts = Counter(patient__code_values)
        patients_data = []
        for pt_code,no_of_sessions in patient__code_counts.items():
            pt_data = ps_items.filter(session__patient__code=pt_code).first()
            pt_obj = {"pt_code":pt_code,"pt_id":pt_data["session__patient_id"],"pt_name":pt_data["session__patient__fullname"],"no_of_sessions":no_of_sessions}
            patients_data.append(pt_obj)
        return Response(patients_data , status=status.HTTP_200_OK)
    
class DrAllPsiDataApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('screen.view_canceled_visits_cc_report'), login_url='/main/login/'))
    def post(self,request):
        data = request.data
        dr_sessions_ids = data.getlist('dr_sessions[]')
        session_status = SessionStatus.objects.all().values("code","color_code").order_by("code")
        ps_items = PatientSessionItem.objects.filter(id__in=dr_sessions_ids).values("session__patient__fullname","session__patient_id","session__patient__code","id","session_status_id",)
        patient_ids_values = list(ps_items.values_list("session__patient_id", flat=True))
        patient_ids_counts = Counter(patient_ids_values)
        patients_data = []
        for pt_id,no_of_sessions in patient_ids_counts.items():
            pt_sessions = ps_items.filter(session__patient_id=pt_id)
            pt_session_obj = pt_sessions.first()
            pt_obj = {"pt_code":pt_session_obj["session__patient__code"],"pt_id":pt_session_obj["session__patient_id"],"pt_name":pt_session_obj["session__patient__fullname"],"no_of_sessions":no_of_sessions}
            pt_sessions_detail_list = []
            #pdb.set_trace()
            for status_obj in session_status :
                #status_obj_info = {}
                status_obj_code = status_obj["code"]
                #status_obj_color_code = status_obj["color_code"]
                status_obj_count = pt_sessions.filter(session_status_id=status_obj_code).values('id').aggregate(count_sessions=Coalesce(Count('id'),0))['count_sessions']
                pt_sessions_detail_list.append(status_obj_count)
            pt_obj["sessions_detail_list"] = pt_sessions_detail_list
            patients_data.append(pt_obj)
        return Response(patients_data , status=status.HTTP_200_OK)



class drCanceldPsiDataApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('screen.view_canceled_visits_cc_report'), login_url='/main/login/'))
    def post(self,request):
        data = request.data
        dr_sessions_ids = data.getlist('dr_sessions[]')
        ps_items = PatientSessionItem.objects.filter(id__in=dr_sessions_ids).values("session__patient__fullname","session__patient_id","session__patient__code")
        patient__code_values = list(ps_items.values_list("session__patient__code", flat=True))
        patient__code_counts = Counter(patient__code_values)
        patients_data = []
        for pt_code,no_of_sessions in patient__code_counts.items():
            pt_data = ps_items.filter(session__patient__code=pt_code).first()
            pt_obj = {"pt_code":pt_code,"pt_id":pt_data["session__patient_id"],"pt_name":pt_data["session__patient__fullname"],"no_of_sessions":no_of_sessions}
            patients_data.append(pt_obj)
        return Response(patients_data , status=status.HTTP_200_OK)

class PtPrognosisDataApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_PtPrognosisData'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        all_doctors = User.objects.filter(user_category="2DR",is_active=True)
        for branch in branches:
            branch["doctors"] = all_doctors.filter(groups__branch_id=branch["id"])
        all_pt_status=PatientStatus.objects.all().order_by('ps_order')
        session_status = SessionStatus.objects.all().order_by("code")
        return render(request,"Clinic/clreportApp/pt_prognosis_data.html",
                      {"branches":branches,"all_pt_status":all_pt_status ,"session_status":session_status, 'activePage':'PtPrognosisData'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_PtPrognosisData'), login_url='/auth/login/'))
    def post(self,request):
        data=request.data
        branch_ids = data.getlist('branch_ids[]')
        status_ids = data.getlist('status_ids[]') 
        #branch_id = data["branch_id"]
        #doctor_id = data["doctor_id"]
        #status_ids = data["status_id"]
        q_sessions_list = []
        if branch_ids : all_doctors = User.objects.filter(user_category="2DR",groups__branch_id__in=branch_ids,is_active=True).values("id","name")
        else : all_doctors = User.objects.filter(user_category="2DR",is_active=True).values("id","name")
        
        # if doctor_id != "0":
        #     branch_doctors = all_doctors.filter(id=doctor_id).values("id","name")
        # else :
        #     branch_doctors = all_doctors
        # q_sessions_list.append(Q(doctor_id__in=branch_doctors.values_list('id',flat=True)))
        q_sessions_list.append(Q(doctor_id__in=set(all_doctors.values_list('id',flat=True))))
        startDateTime,endDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        q_pt_info_list = [Q(patient__created_at__range=[startDateTime,endDateTime])]
        #pdb.set_trace()
        if branch_ids : q_pt_info_list.append(Q(patient__branch_id__in=branch_ids))
        #q_visits_list = []
        #patients_ids = patients.values_list("patient_id",flat=True)
        #q_sessions_list.append(Q(session__patient_id__in=patients_ids))
        #q_sessions_list.append(Q(session__patient_id__in=patients_ids))
        #هفلتر المرضى اللى عملوا جلسات فقط مع الدكتور ده 
        patients_ids = PatientSessionItem.objects.filter(reduce(operator.and_, q_sessions_list)).values_list("session__patient_id",flat=True)
        psi_items = PatientSessionItem.objects.filter(session__patient_id__in=set(patients_ids)).values("session__patient_id","doctor_id","session_status_id")
        #patients_ids = psi_items.values_list("session__patient_id",flat=True)
        q_pt_info_list.append(Q(patient_id__in=set(patients_ids)))
        #if status_id != "0":q_pt_info_list.append(Q(pt_status_id=status_id))
        if status_ids :q_pt_info_list.append(Q(pt_status_id__in=status_ids))
        patients = PatientInfo.objects.filter(reduce(operator.and_, q_pt_info_list)).order_by('patient__branch_id','pt_status_id').values("patient_id","patient__fullname","patient__number_1","patient__branch__name","last_visit","last_session","pt_status__name","patient__dr_notes")
        session_status = SessionStatus.objects.all().values("code","color_code").order_by("code")
        response_data =[]
        date_format = "%Y-%m-%d"
        for pt in patients:
            pt_sessions = psi_items.filter(session__patient_id=pt["patient_id"])
            attend_sessions_count_by_dr = pt_sessions.filter(session_status_id="2AT").values('doctor__name').annotate(count_sessions=Coalesce(Count('doctor_id'),0))
            pt_sessions_attend_list = []
            if attend_sessions_count_by_dr :
                for dr_obj in attend_sessions_count_by_dr:
                    pt_sessions_attend_list.append({"dr_name":dr_obj["doctor__name"],"sessions_count":dr_obj["count_sessions"]})
            pt["pt_sessions_attend_list"] = pt_sessions_attend_list
            pt_sessions_total = pt_sessions.count()
            pt["pt_sessions_total"] = pt_sessions_total
            pt_sessions_detail_list = []
            for status_obj in session_status :
                status_obj_code = status_obj["code"]
                #status_obj_color_code = status_obj["color_code"]
                status_obj_count = pt_sessions.filter(session_status_id=status_obj_code).values('id').aggregate(count_sessions=Coalesce(Count('id'),0))['count_sessions']
                pt_sessions_detail_list.append(status_obj_count)
            pt["pt_sessions_detail_list"] = pt_sessions_detail_list
            if pt["last_visit"] : pt["last_visit"] = pt["last_visit"].strftime(date_format)
            if pt["last_session"] : pt["last_session"] = pt["last_session"].strftime(date_format)
            response_data.append(pt)
            
        return Response(response_data , status=status.HTTP_200_OK)


class PtPrognosisDataCustomApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_PtPrognosisData'), login_url='/auth/login/'))
    def post(self,request):
        data=request.data
        branch_id = data["branch_id"]
        status_ids = data.getlist('status_ids[]') 
        startDateTime,endDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        q_pt_info_list = [Q(patient__created_at__range=[startDateTime,endDateTime])]
        q_pt_info_list.append(Q(patient__branch_id=branch_id))
        if status_ids :q_pt_info_list.append(Q(pt_status_id__in=status_ids))
        patients = PatientInfo.objects.filter(reduce(operator.and_, q_pt_info_list)).order_by('pt_status_id').values("patient_id","patient__fullname","patient__number_1","last_visit","last_session","pt_status__name","patient__dr_notes")
        response_data =[]
        date_format = "%Y-%m-%d"
        for pt in patients:
            if pt["last_visit"] : pt["last_visit"] = pt["last_visit"].strftime(date_format)
            if pt["last_session"] : pt["last_session"] = pt["last_session"].strftime(date_format)
            response_data.append(pt)
        return Response(response_data , status=status.HTTP_200_OK)
from brpatientApp.models import Patient,IrregularPtData
class IrregularPtDataReportApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('ClManagement.view_IrregularPtData'), login_url='/auth/login/'))
    def get(self,request):
        branches = Branch.objects.filter(is_active=True).values('id','name')
        return render(request,"Clinic/clreportApp/irregular_pt_data.html",{"branches":branches,"page_source":"Clinic", 'activePage':'IrregularPtData'})
    def post(self,request):
        data=request.data
        branch_id = data["branch_id"]
        is_hidden_input = True if data["is_hidden"]=="true" else False
        no_of_sessions_for_irregular = 3
        patients_with_limited_sessions = Patient.objects.annotate(
            session_item_count=Count('PatientSession_patient__PatientSessionItem_session',
            filter=Q(PatientSession_patient__PatientSessionItem_session__session_status__code="2AT"))
            ).filter(
                #IrregularPtInfo_patient__is_hidden=is_hidden,
                session_item_count__lte=no_of_sessions_for_irregular,
                session_item_count__gt=0,
                branch_id=branch_id).values('id','code','fullname','number_1','session_item_count')

                #branch_id=branch_id).order_by('session_item_count').values('id','code','fullname','number_1','session_item_count')
        #print("Done Filter Patients")
        patients_with_limited_sessions_ids = patients_with_limited_sessions.values_list('id',flat=True)
        irregular_pt_data = IrregularPtData.objects.filter(patient_id__in=patients_with_limited_sessions_ids)
        #irregular_pt__ids = irregular_pt_data.values_list('patient_id',flat=True)
        # if user want hidden we only get patient hidden before 
        # if user want current get not all patient , and fetch data for irregular entered before
        hidden_only__ids = irregular_pt_data.filter(is_hidden=True).values_list('patient_id',flat=True)
        #print(patients_with_limited_sessions)
        if is_hidden_input == True : 
            patients_after_filter_irregular  = patients_with_limited_sessions.filter(id__in=hidden_only__ids)
        else :
            patients_after_filter_irregular = patients_with_limited_sessions.exclude(id__in=hidden_only__ids)

        patients_after_filter_irregular__ids = patients_after_filter_irregular.values_list('id',flat=True)
        #print("Done Filter Patients Irregular ")
        psi_items = PatientSessionItem.objects.filter(session__patient_id__in=patients_after_filter_irregular__ids).values("doctor__name","doctor_id","session_day")
        # Assuming `today` is a date object representing the current date
        today = timezone.now().date() 
        # Calculate the date 4 days before today
        number_of_days_to_exclude = 4
        date_days_before = today - timedelta(days=number_of_days_to_exclude)
        psi_items_within_last_days = psi_items.filter(session_day__gte=date_days_before)
        psi_last_days_pt__ids = psi_items_within_last_days.values_list('session__patient_id',flat=True)
        #filter patient data again to execlude earilier patient sessions 
        patients_after_filter_irregular_filter_days = patients_after_filter_irregular.exclude(id__in=psi_last_days_pt__ids)
        #patients_with_latest_session = patients_after_filter_irregular_filter_days.annotate(
        patients_with_oldest_session = patients_after_filter_irregular_filter_days.annotate(
        oldest_session_day=Min('PatientSession_patient__PatientSessionItem_session__session_day')
        #latest_session_day=Max('PatientSession_patient__PatientSessionItem_session__session_day')
        )
        # Order the patients by the latest_session_day in descending order
        #patients_after_filter_irregular_filter_days_ordered = patients_with_latest_session.order_by('-latest_session_day')
        patients_after_filter_irregular_filter_days_ordered = patients_with_oldest_session.order_by('oldest_session_day')
        response_data =[]
        #patients_data_result = patients_with_limited_sessions if not is_hidden else patients_with_limited_sessions.filter(id__in=irregular_pt__ids)
        date_format = "%Y-%m-%d"
        #print(f"Done Get Sessions before  {patients_after_filter_irregular.count()}")
        #print(f"Done Get Sessions after  {patients_after_filter_irregular_filter_days.count()}")
        #print(patients_after_filter_irregular.count())
        counter = 0 
        for pt_obj in patients_after_filter_irregular_filter_days_ordered:
            counter = counter +1 
            #print(f"Start Get Sessions ",pt_obj["id"] , "counter :" , counter)
            pt = {"id":pt_obj["id"],"code":pt_obj["code"],"fullname":pt_obj["fullname"],"number_1":pt_obj["number_1"],"session_item_count":pt_obj["session_item_count"],}
            pt_sessions = psi_items.filter(session__patient_id=pt["id"])
            irregular_pt = irregular_pt_data.filter(patient_id=pt["id"])
            irregular_pt_obj = irregular_pt[0] if irregular_pt else None
            pt_sessions_attend = pt_sessions.filter(session_status_id="2AT")
            attend_sessions_count_by_dr = pt_sessions_attend.values('doctor__name').annotate(count_sessions=Coalesce(Count('doctor_id'),0))
            pt_sessions_attend_doctors = []
            if attend_sessions_count_by_dr :
                for dr_obj in attend_sessions_count_by_dr:
                    pt_sessions_attend_doctors.append({"dr_name":dr_obj["doctor__name"],"sessions_count":dr_obj["count_sessions"]})
            pt["pt_sessions_attend_doctors"] = pt_sessions_attend_doctors
            pt["attend_session_days"] = list(pt_sessions_attend.values_list('session_day',flat=True))
            
            pt_sessions_total = pt_sessions.count()
            pt["pt_sessions_total"] = pt_sessions_total
            pt_sessions_attend = pt_sessions_attend.count()
            pt["pt_sessions_attend"] = pt_sessions_attend
            #import pdb
            #pdb.set_trace()
            if irregular_pt_obj :
                pt["br_notes"] = irregular_pt_obj.br_notes
                pt["br_created_by__name"] = irregular_pt_obj.br_created_by.name if irregular_pt_obj.br_created_by else ""
                pt["br_created_at__date"] = irregular_pt_obj.br_created_at.strftime(date_format) if irregular_pt_obj.br_created_at else ""
                pt["cl_notes"] = irregular_pt_obj.cl_notes if irregular_pt_obj.cl_notes else ""
            else :
                pt["br_notes"] = None 
                pt["br_created_by__name"] = None 
                pt["br_created_at__date"] = None
                pt["cl_notes"] = None 

            response_data.append(pt)
        
        return Response(response_data , status=status.HTTP_200_OK)
'''
        import pdb
        pdb.set_trace()

        #).filter(session_item_count__lte=3,branch_id=branch_id,PatientSession_patient__PatientSessionItem__session_status="2AT").values_list('id',flat=True)
        patients_with_limited_sessions_ids = Patient.objects.annotate(
            session_item_count=Count('PatientSession_patient__PatientSessionItem_session',filter=Q(PatientSession_patient__PatientSessionItem_session__session_status__code="2AT"))
        ).filter(session_item_count__lte=6,branch_id=branch_id).values('id', 'fullname', 'session_item_count',"branch_id")
        
        #).filter(session_item_count__in=[1,2,3,4,5,6],branch_id=branch_id).values_list('session_item_count',flat=True)
        #patient_ids = patients_with_limited_sessions.values_list('id',flat=True) 
        PatientSessionItem.objects.filter(session__patient_id__in=patients_with_limited_sessions_ids).values('session__patient__fullname')
        
        q_sessions_list = []
        #PatientSessionItem.objects.annotate(session_item_count=Count('session__patient__fullname'))[0]
        #all PatientSessionItem should be retrived 

        q_sessions_list.append(Q(doctor_id__in=set(all_doctors.values_list('id',flat=True))))
        startDateTime,endDateTime = getStartEndDateTime(data["startDate"],data["endDate"])
        q_pt_info_list = [Q(patient__created_at__range=[startDateTime,endDateTime])]
        #pdb.set_trace()
        if branch_ids : q_pt_info_list.append(Q(patient__branch_id__in=branch_ids))
        #q_visits_list = []
        #patients_ids = patients.values_list("patient_id",flat=True)
        #q_sessions_list.append(Q(session__patient_id__in=patients_ids))
        #q_sessions_list.append(Q(session__patient_id__in=patients_ids))
        #هفلتر المرضى اللى عملوا جلسات فقط مع الدكتور ده 
        patients_ids = PatientSessionItem.objects.filter(reduce(operator.and_, q_sessions_list)).values_list("session__patient_id",flat=True)
        psi_items = PatientSessionItem.objects.filter(session__patient_id__in=set(patients_ids)).values("session__patient_id","doctor_id","session_status_id")
        #patients_ids = psi_items.values_list("session__patient_id",flat=True)
        q_pt_info_list.append(Q(patient_id__in=set(patients_ids)))
        #if status_id != "0":q_pt_info_list.append(Q(pt_status_id=status_id))
        if status_ids :q_pt_info_list.append(Q(pt_status_id__in=status_ids))
        patients = PatientInfo.objects.filter(reduce(operator.and_, q_pt_info_list)).order_by('patient__branch_id','pt_status_id').values("patient_id","patient__fullname","patient__branch__name","last_visit","last_session","pt_status__name","patient__dr_notes")
        session_status = SessionStatus.objects.all().values("code","color_code").order_by("code")
        response_data =[]
        date_format = "%Y-%m-%d"
        for pt in patients:
            pt_sessions = psi_items.filter(session__patient_id=pt["patient_id"])
            attend_sessions_count_by_dr = pt_sessions.filter(session_status_id="2AT").values('doctor__name').annotate(count_sessions=Coalesce(Count('doctor_id'),0))
            pt_sessions_attend_list = []
            if attend_sessions_count_by_dr :
                for dr_obj in attend_sessions_count_by_dr:
                    pt_sessions_attend_list.append({"dr_name":dr_obj["doctor__name"],"sessions_count":dr_obj["count_sessions"]})
            pt["pt_sessions_attend_list"] = pt_sessions_attend_list
            pt_sessions_total = pt_sessions.count()
            pt["pt_sessions_total"] = pt_sessions_total
            pt_sessions_detail_list = []
            for status_obj in session_status :
                status_obj_code = status_obj["code"]
                #status_obj_color_code = status_obj["color_code"]
                status_obj_count = pt_sessions.filter(session_status_id=status_obj_code).values('id').aggregate(count_sessions=Coalesce(Count('id'),0))['count_sessions']
                pt_sessions_detail_list.append(status_obj_count)
            pt["pt_sessions_detail_list"] = pt_sessions_detail_list
            if pt["last_visit"] : pt["last_visit"] = pt["last_visit"].strftime(date_format)
            if pt["last_session"] : pt["last_session"] = pt["last_session"].strftime(date_format)
            response_data.append(pt)
            
        return Response(response_data , status=status.HTTP_200_OK)
'''